@@ -16,7 +16,9 @@
 #include <string.h>
 #include <stdarg.h>
 
+#if (!defined IOS) && (!defined QT_CONFIG)
 #include <gif_lib.h>
+#endif //IOS
 
 #include "JTypes.h"
 
@@ -25,7 +27,7 @@
 
 	#include <windows.h>
 
-#elif defined (LINUX)
+#elif defined (LINUX) || defined(IOS)
 
 #else
 
@@ -73,11 +75,12 @@ protected:
 	void InitRenderer();
 	void DestroyRenderer();
 
-
+  float mActualWidth;
+  float mActualHeight;
 
 public:
 
-#if defined (WIN32) || defined (LINUX)
+#if defined (WIN32) || defined (LINUX) || defined (IOS)
 #else
   int PixelSize(int textureMode);
 #endif
@@ -141,6 +144,16 @@ public:
 	//////////////////////////////////////////////////////////////////////////
 	JTexture* LoadTexture(const char* filename, int mode = 0, int textureFormat = TEXTURE_FORMAT);
 
+    /*
+    ** Helper function - on Win, LoadTexture previously performed the image transfer into GL memory.
+    ** However, this doesn't work if you want to call LoadTexture in a separate worker thread, as 
+    ** OpenGL has a limitation where only one thread can run the context. Now, the image loading/decompression
+    ** happens in LoadTexture, and the JQuad constructor will complete the texture binding with this helper function.
+    **
+    ** On PSP, this is a no-op.
+    */
+    void TransferTextureToGLContext(JTexture& inTexture);
+
 	//////////////////////////////////////////////////////////////////////////
 	/// Create texture from memory on the fly.
 	///
@@ -524,6 +537,29 @@ public:
 	//////////////////////////////////////////////////////////////////////////
 	void SetImageFilter(JImageFilter* imageFilter);
 
+  /**
+  ** Set/Get methods for the actual display screen size.
+  */
+  inline void SetActualWidth(float inWidth)
+  {
+    mActualWidth = inWidth;
+  }
+
+  inline void SetActualHeight(float inHeight)
+  {
+    mActualHeight = inHeight;
+  }
+
+  inline float GetActualWidth()
+  {
+    return mActualWidth;
+  }
+
+  inline float GetActualHeight()
+  {
+    return mActualHeight;
+  }
+
 private:
 
 	struct TextureInfo
@@ -536,16 +572,17 @@ private:
 		bool mVRAM;
 	};
 
+#if (!defined IOS) && (!defined QT_CONFIG)
 	void LoadJPG(TextureInfo &textureInfo, const char *filename, int mode = 0, int TextureFormat = TEXTURE_FORMAT);
 	int LoadPNG(TextureInfo &textureInfo, const char *filename, int mode = 0, int TextureFormat = TEXTURE_FORMAT);
 	void LoadGIF(TextureInfo &textureInfo, const char *filename, int mode = 0, int TextureFormat = TEXTURE_FORMAT);
 	int image_readgif(void * handle, TextureInfo &textureInfo, DWORD * bgcolor, InputFunc readFunc,int mode = 0, int TextureFormat = TEXTURE_FORMAT);
-
+#endif //(IOS)
+	
 	static JRenderer* mInstance;
 
-
-#if defined (WIN32) || defined (LINUX)
-
+#if defined (WIN32) || defined (LINUX) || defined (IOS)
+	
 	GLuint mCurrentTex;
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0) || (defined WIN32)
   // MVP matrix
