@@ -9,10 +9,9 @@
 //-------------------------------------------------------------------------------------
 #define GL_GLEXT_PROTOTYPES
 
+#if (!defined IOS) && (!defined QT_CONFIG)
 #ifdef WIN32
   #pragma warning(disable : 4786)
-  extern int actualWidth;
-  extern int actualHeight;
   #pragma comment( lib, "giflib.lib" )
 #endif
 
@@ -28,7 +27,7 @@ extern "C" {
 #ifdef __cplusplus
 }
 #endif
-
+#endif //IOS
 
 #include "../../include/JGE.h"
 #include "../../include/JRenderer.h"
@@ -42,6 +41,17 @@ extern "C" {
 #endif
 #endif
 
+#ifdef _DEBUG
+#define checkGlError()            \
+{                                 \
+    GLenum glError = glGetError();  \
+    if(glError != 0)                \
+    printf("%s : %u : GLerror is %u\n", __FUNCTION__, __LINE__, glError); \
+}
+#else
+#define checkGlError() (void(0))
+#endif
+
 //#define FORCE_GL2
 #ifdef FORCE_GL2
 // This code is to force the windows code to use GL_VERSION_2_0 even if it's not defined in the header files
@@ -277,8 +287,8 @@ static glslFunctions g_glslfuncts;
 JQuad::JQuad(JTexture *tex, float x, float y, float width, float height)
 		:mTex(tex), mX(x), mY(y), mWidth(width), mHeight(height)
 {
-
 	JASSERT(tex != NULL);
+    JRenderer::GetInstance()->TransferTextureToGLContext(*tex);
 
 	mHotSpotX = 0.0f;
 	mHotSpotY = 0.0f;
@@ -290,7 +300,6 @@ JQuad::JQuad(JTexture *tex, float x, float y, float width, float height)
 	mVFlipped = false;
 
 	SetTextureRect(x, y, width, height);
-
 }
 
 void JQuad::SetTextureRect(float x, float y, float w, float h)
@@ -304,7 +313,6 @@ void JQuad::SetTextureRect(float x, float y, float w, float h)
 	mTY0 = y/mTex->mTexHeight;
 	mTX1 = (x+w)/mTex->mTexWidth;
 	mTY1 = (y+h)/mTex->mTexHeight;
-
 }
 
 
@@ -331,23 +339,34 @@ void JQuad::SetHotSpot(float x, float y)
 
 //////////////////////////////////////////////////////////////////////////
 
-JTexture::JTexture()
+JTexture::JTexture() : mBuffer(NULL)
 {
 	mTexId = -1;
 }
 
 JTexture::~JTexture()
 {
-        if (mTexId != (GLuint)-1)
-		glDeleteTextures(1, &mTexId);
+    checkGlError();
+    if (mTexId != (GLuint)-1)
+        glDeleteTextures(1, &mTexId);
+    checkGlError();
+
+    if (mBuffer)
+    {
+#ifndef QT_CONFIG
+        delete [] mBuffer;
+#endif
+        mBuffer = NULL;
+    }
 }
 
 
 void JTexture::UpdateBits(int x, int y, int width, int height, PIXEL_TYPE* bits)
 {
-	JRenderer::GetInstance()->BindTexture(this);
+  checkGlError();
+  JRenderer::GetInstance()->BindTexture(this);
 	glTexSubImage2D(GL_TEXTURE_2D, 0, x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, bits);
-
+  checkGlError();
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -382,7 +401,7 @@ void JRenderer::Destroy()
 	}
 }
 
-JRenderer::JRenderer()
+JRenderer::JRenderer() : mActualWidth(SCREEN_WIDTH_F), mActualHeight(SCREEN_HEIGHT_F)
 {
 }
 
@@ -568,6 +587,7 @@ void esOrtho(ESMatrix *result, float left, float right, float bottom, float top,
 //
 GLuint esLoadShader ( GLenum type, const char *shaderSrc )
 {
+  checkGlError();
    GLuint shader;
    GLint compiled;
 
@@ -606,6 +626,7 @@ GLuint esLoadShader ( GLenum type, const char *shaderSrc )
       return 0;
    }
 
+   checkGlError();
    return shader;
 
 }
@@ -619,6 +640,8 @@ GLuint esLoadShader ( GLenum type, const char *shaderSrc )
 //
 GLuint esLoadProgram ( const char *vertShaderSrc, const char *fragShaderSrc )
 {
+  checkGlError();
+
    GLuint vertexShader;
    GLuint fragmentShader;
    GLuint programObject;
@@ -675,6 +698,8 @@ GLuint esLoadProgram ( const char *vertShaderSrc, const char *fragShaderSrc )
    glDeleteShader ( vertexShader );
    glDeleteShader ( fragmentShader );
 
+   checkGlError();
+
    return programObject;
 }
 
@@ -682,7 +707,8 @@ GLuint esLoadProgram ( const char *vertShaderSrc, const char *fragShaderSrc )
 
 void JRenderer::InitRenderer()
 {
-	mCurrentTextureFilter = TEX_FILTER_NONE;
+  checkGlError();
+  mCurrentTextureFilter = TEX_FILTER_NONE;
 	mImageFilter = NULL;
 
 	mCurrTexBlendSrc = BLEND_SRC_ALPHA;
@@ -773,10 +799,12 @@ void JRenderer::InitRenderer()
   // Get the sampler location
   prog2_samplerLoc = glGetUniformLocation ( prog2, "s_texture" );
 #endif //(defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
+  checkGlError();
 }
 
 void JRenderer::DestroyRenderer()
 {
+  checkGlError();
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
     // Delete program object
     glDeleteProgram ( prog1 );
@@ -794,26 +822,29 @@ void JRenderer::BeginScene()
   esOrtho(&theMvpMatrix, 0.0f, SCREEN_WIDTH_F, 0.0f, SCREEN_HEIGHT_F-1.0f,-1.0f, 1.0f);
 #endif //(!defined GL_ES_VERSION_2_0) && (!defined GL_VERSION_2_0)
 #ifdef WIN32
-  float scaleH = (float)actualHeight/SCREEN_HEIGHT_F;
-  float scaleW = (float)actualWidth/SCREEN_WIDTH_F;
+  float scaleH = mActualHeight/SCREEN_HEIGHT_F;
+  float scaleW = mActualWidth/SCREEN_WIDTH_F;
   glScalef(scaleW,scaleW,1.f);
 #endif
+  checkGlError();
 }
 
 
 void JRenderer::EndScene()
 {
-	glFlush ();
+  checkGlError();
+  glFlush ();
+  checkGlError();
 }
 
 void JRenderer::BindTexture(JTexture *tex)
 {
-
-	if (mCurrentTex != tex->mTexId)
+  checkGlError();
+  if (mCurrentTex != tex->mTexId)
 	{
 		mCurrentTex = tex->mTexId;
 
-		glBindTexture(GL_TEXTURE_2D, tex->mTexId);
+		glBindTexture(GL_TEXTURE_2D, mCurrentTex);
 
 		//if (mCurrentTextureFilter != tex->mFilter)
 		{
@@ -830,6 +861,7 @@ void JRenderer::BindTexture(JTexture *tex)
 			}
 		}
 	}
+  checkGlError();
 }
 
 
@@ -852,7 +884,9 @@ void Swap(float *a, float *b)
 
 void JRenderer::RenderQuad(JQuad* quad, float xo, float yo, float angle, float xScale, float yScale)
 {
-	//yo = SCREEN_HEIGHT-yo-1;//-(quad->mHeight);
+  checkGlError();
+
+  //yo = SCREEN_HEIGHT-yo-1;//-(quad->mHeight);
 	float width = quad->mWidth;
 	float height = quad->mHeight;
 	float x = -quad->mHotSpotX;
@@ -1019,12 +1053,13 @@ void JRenderer::RenderQuad(JQuad* quad, float xo, float yo, float angle, float x
         glColor4ub(255, 255, 255, 255);
 #endif //(defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-
+  checkGlError();
 }
 
 
 void JRenderer::RenderQuad(JQuad* quad, VertexColor* pt)
 {
+  checkGlError();
 
 	for (int i=0;i<4;i++)
 	{
@@ -1117,17 +1152,19 @@ void JRenderer::RenderQuad(JQuad* quad, VertexColor* pt)
 
 	//glDisable(GL_BLEND);
 
+  checkGlError();
 }
 
 
 void JRenderer::FillRect(float x, float y, float width, float height, PIXEL_TYPE color)
 {
-	y = SCREEN_HEIGHT_F - y - height;
+  checkGlError();
+
+  y = SCREEN_HEIGHT_F - y - height;
 
 	JColor col;
 	col.color = color;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   GLfloat vVertices[] = {
       x,        y+height, 0.0f,
@@ -1161,6 +1198,7 @@ void JRenderer::FillRect(float x, float y, float width, float height, PIXEL_TYPE
 
   glDrawArrays(GL_TRIANGLE_STRIP,0,4);
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
 
 	glBegin(GL_QUADS);
@@ -1179,21 +1217,23 @@ void JRenderer::FillRect(float x, float y, float width, float height, PIXEL_TYPE
 	glEnd();
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //(defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
 
+  checkGlError();
 }
 
 
 void JRenderer::DrawRect(float x, float y, float width, float height, PIXEL_TYPE color)
 {
-	y = SCREEN_HEIGHT_F - y - height;
+  checkGlError();
+
+  y = SCREEN_HEIGHT_F - y - height;
 
 	JColor col;
 	col.color = color;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   GLfloat vVertices[] = {
       x,        y,        0.0f,
@@ -1227,6 +1267,7 @@ void JRenderer::DrawRect(float x, float y, float width, float height, PIXEL_TYPE
 
   glDrawArrays(GL_LINE_LOOP,0,4);
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_LINES);
 
@@ -1246,9 +1287,10 @@ void JRenderer::DrawRect(float x, float y, float width, float height, PIXEL_TYPE
 
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (!defined GL_ES_VERSION_2_0) && (!defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 
@@ -1264,9 +1306,8 @@ void JRenderer::FillRect(float x, float y, float width, float height, PIXEL_TYPE
 
 void JRenderer::FillRect(float x, float y, float width, float height, JColor* colors)
 {
-	y = SCREEN_HEIGHT_F - y - height;
-
-	glDisable(GL_TEXTURE_2D);
+  checkGlError();
+  y = SCREEN_HEIGHT_F - y - height;
 
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   GLfloat vVertices[] = {
@@ -1301,6 +1342,7 @@ void JRenderer::FillRect(float x, float y, float width, float height, JColor* co
 
   glDrawArrays(GL_TRIANGLE_STRIP,0,4);
 #else
+  glDisable(GL_TEXTURE_2D);
   glBegin(GL_QUADS);
 		// top left corner
 		glColor4ub(colors[0].r, colors[0].g, colors[0].b, colors[0].a);
@@ -1322,17 +1364,18 @@ void JRenderer::FillRect(float x, float y, float width, float height, JColor* co
 
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
 	//glDisable(GL_BLEND);
+  checkGlError();
 }
 
 
 void JRenderer::DrawLine(float x1, float y1, float x2, float y2, PIXEL_TYPE color)
 {
+  checkGlError();
 //	glLineWidth (mLineWidth);
-	glDisable(GL_TEXTURE_2D);
 	JColor col;
 	col.color = color;
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
@@ -1364,20 +1407,23 @@ void JRenderer::DrawLine(float x1, float y1, float x2, float y2, PIXEL_TYPE colo
 
   glDrawArrays(GL_LINES,0,2);
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_LINES);
 		glVertex2f(x1, SCREEN_HEIGHT_F-y1);
 		glVertex2f(x2, SCREEN_HEIGHT_F-y2);
 	glEnd();
   glColor4ub(255, 255, 255, 255);
-#endif //#if (!defined GL_ES_VERSION_2_0) && (!defined GL_VERSION_2_0)
   glEnable(GL_TEXTURE_2D);
+#endif //#if (!defined GL_ES_VERSION_2_0) && (!defined GL_VERSION_2_0)
+  checkGlError();
 }
 
 
 void JRenderer::Plot(float x, float y, PIXEL_TYPE color)
 {
-	glDisable(GL_TEXTURE_2D);
+  checkGlError();
+  glDisable(GL_TEXTURE_2D);
 	JColor col;
 	col.color = color;
 #if (!defined GL_ES_VERSION_2_0) && (!defined GL_VERSION_2_0)
@@ -1390,12 +1436,14 @@ void JRenderer::Plot(float x, float y, PIXEL_TYPE color)
   // FIXME, not used
 #endif //#if (!defined GL_ES_VERSION_2_0) && (!defined GL_VERSION_2_0)
   glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 
 void JRenderer::PlotArray(float *x, float *y, int count, PIXEL_TYPE color)
 {
-	glDisable(GL_TEXTURE_2D);
+  checkGlError();
+  glDisable(GL_TEXTURE_2D);
 	JColor col;  
 	col.color = color;
 #if (!defined GL_ES_VERSION_2_0) && (!defined GL_VERSION_2_0)
@@ -1409,6 +1457,7 @@ void JRenderer::PlotArray(float *x, float *y, int count, PIXEL_TYPE color)
   // FIXME, not used
 #endif //#if (!defined GL_ES_VERSION_2_0) && (!defined GL_VERSION_2_0)
   glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 
@@ -1430,7 +1479,7 @@ static int getNextPower2(int width)
 }
 
 
-
+#if (!defined IOS) && (!defined QT_CONFIG)
 static void jpg_null(j_decompress_ptr cinfo __attribute__((unused)))
 {
 }
@@ -1610,110 +1659,49 @@ static void PNGCustomReadDataFn(png_structp png_ptr, png_bytep data, png_size_t
    }
 }
 
-
 JTexture* JRenderer::LoadTexture(const char* filename, int mode, int TextureFormat __attribute__((unused)))
 {
-	TextureInfo textureInfo;
+    TextureInfo textureInfo;
 
-	textureInfo.mBits = NULL;
+    textureInfo.mBits = NULL;
 
-	if (strstr(filename, ".jpg")!=NULL || strstr(filename, ".JPG")!=NULL)
-		LoadJPG(textureInfo, filename);
-	else if(strstr(filename, ".gif")!=NULL || strstr(filename, ".GIF")!=NULL)
-		LoadGIF(textureInfo,filename);
-	else if(strstr(filename, ".png")!=NULL || strstr(filename, ".PNG")!=NULL)
-		LoadPNG(textureInfo, filename);
+    if (strstr(filename, ".jpg")!=NULL || strstr(filename, ".JPG")!=NULL)
+        LoadJPG(textureInfo, filename);
+    else if(strstr(filename, ".gif")!=NULL || strstr(filename, ".GIF")!=NULL)
+        LoadGIF(textureInfo,filename);
+    else if(strstr(filename, ".png")!=NULL || strstr(filename, ".PNG")!=NULL)
+        LoadPNG(textureInfo, filename);
 
-        if (textureInfo.mBits == NULL) {
-            printf("Texture %s failed to load\n", filename);
-            return NULL;
-        }
+    if (textureInfo.mBits == NULL) {
+        printf("Texture %s failed to load\n", filename);
+        return NULL;
+    }
 
-	bool ret = false;
+    bool ret = false;
 
-	JTexture *tex = new JTexture();
+    JTexture *tex = new JTexture();
 
- 	if (tex)
-	{
-		if (mImageFilter != NULL)
-			mImageFilter->ProcessImage((PIXEL_TYPE*)textureInfo.mBits, textureInfo.mWidth, textureInfo.mHeight);
+    if (tex)
+    {
+        if (mImageFilter != NULL)
+            mImageFilter->ProcessImage((PIXEL_TYPE*)textureInfo.mBits, textureInfo.mWidth, textureInfo.mHeight);
 
-		tex->mFilter = TEX_FILTER_LINEAR;
-		tex->mWidth = textureInfo.mWidth;
-		tex->mHeight = textureInfo.mHeight;
-		tex->mTexWidth = textureInfo.mTexWidth;
-		tex->mTexHeight = textureInfo.mTexHeight;
+        tex->mFilter = TEX_FILTER_LINEAR;
+        tex->mWidth = textureInfo.mWidth;
+        tex->mHeight = textureInfo.mHeight;
+        tex->mTexWidth = textureInfo.mTexWidth;
+        tex->mTexHeight = textureInfo.mTexHeight;
 
-		GLuint texid;
-		glGenTextures(1, &texid);
-		tex->mTexId = texid;
-                GLenum glError = glGetError();
-
-                if (/*texid*/ glError == 0)
-		{
-
-			// OpenGL texture has (0,0) at lower-left
-			// Pay attention when doing texture mapping!!!
-
-                            glBindTexture(GL_TEXTURE_2D, texid);    // Bind To The Texture ID
-
-/* NOT USED
-			if (mode == TEX_TYPE_MIPMAP)			// generate mipmaps
-			{
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
-				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
-                                gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, textureInfo.mTexWidth, textureInfo.mTexHeight, GL_RGBA, GL_UNSIGNED_BYTE, textureInfo.mBits);
-                        }
-			else if (mode == TEX_TYPE_SKYBOX)		// for skybox
-			{
-#define GL_CLAMP_TO_EDGE	0x812F
-
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-                                gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, textureInfo.mTexWidth, textureInfo.mTexHeight, GL_RGBA, GL_UNSIGNED_BYTE, textureInfo.mBits);
-                        }
-			else									// single texture
-*/			{
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
-				glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
-                                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, textureInfo.mTexWidth, textureInfo.mTexHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, textureInfo.mBits);
-                        }
-
-			ret = TRUE;
-
-		}
-                else
-                {
-                    printf("TextureId is 0, GLerror is %u\n", glError);
-                }
-
-
-	}
-
-	delete [] textureInfo.mBits;
-	//delete textureInfo;
-
-	if (!ret)
-	{
-		if (tex)
-			delete tex;
-		tex = NULL;
-	}
-
-	return tex;
+        tex->mBuffer = textureInfo.mBits;
+    }
+ 
+    return tex;
 }
 
 int JRenderer::LoadPNG(TextureInfo &textureInfo, const char *filename, int mode __attribute__((unused)), int TextureFormat __attribute__((unused)))
 {
-
-
 	textureInfo.mBits = NULL;
 
-
 	DWORD* p32;
 
     png_structp png_ptr;
@@ -1998,11 +1986,220 @@ void JRenderer::LoadGIF(TextureInfo &textureInfo, const char *filename, int mode
 	return ;//*/
 }
 
+#elif (defined IOS)
 
+#include <UIKit/UIImage.h>
+
+JTexture* JRenderer::LoadTexture(const char* filename, int mode, int TextureFormat __attribute__((unused)))
+{
+    TextureInfo textureInfo;
+    JTexture *tex = NULL;
+    textureInfo.mBits = NULL;
+    int rawsize = 0;
+    BYTE* rawdata = NULL;
+    JFileSystem* fileSystem = JFileSystem::GetInstance();
+    NSData *texData = NULL;
+    UIImage *image = NULL;
+
+    do {
+        if (!fileSystem->OpenFile(filename))
+                break;
+
+        rawsize = fileSystem->GetFileSize();
+        rawdata = new BYTE[rawsize];
+
+        if (!rawdata)
+        {
+                fileSystem->CloseFile();
+                break;
+        }
+
+        fileSystem->ReadFile(rawdata, rawsize);
+        fileSystem->CloseFile();
+
+        texData = [[NSData alloc] initWithBytes:rawdata length:rawsize];
+        image = [[UIImage alloc] initWithData:texData];
+        CGImageAlphaInfo info;
+        BOOL hasAlpha;
+
+        info = CGImageGetAlphaInfo(image.CGImage);
+        hasAlpha = ((info == kCGImageAlphaPremultipliedLast) || (info == kCGImageAlphaPremultipliedFirst) || (info == kCGImageAlphaLast) || (info == kCGImageAlphaFirst) ? YES : NO);
+
+        if (image == nil) {
+                NSLog(@"Loading Texture : %s failed", filename);
+                break;
+        }
+
+        textureInfo.mWidth = CGImageGetWidth(image.CGImage);
+        textureInfo.mHeight = CGImageGetHeight(image.CGImage);
+        textureInfo.mTexWidth = getNextPower2(textureInfo.mWidth);
+        textureInfo.mTexHeight = getNextPower2(textureInfo.mHeight);
+
+        NSLog(@"Loading Texture : %s : %s : %ux%u", filename, (hasAlpha?"Alpha ":"No Alpha "), textureInfo.mWidth, textureInfo.mHeight);
+
+        textureInfo.mBits = new u8 [ textureInfo.mTexHeight * textureInfo.mTexWidth * 4 ];
+        if (textureInfo.mBits == NULL) {
+                NSLog(@"Texture %s failed to load\n", filename);
+                break;
+        }
+
+        CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
+        info = /*hasAlpha ?*/ kCGImageAlphaPremultipliedLast /*: kCGImageAlphaNoneSkipLast*/;
+
+        CGContextRef context =
+        CGBitmapContextCreate(textureInfo.mBits,
+                                                  textureInfo.mTexWidth,
+                                                  textureInfo.mTexHeight, 8,
+                                                  4 * textureInfo.mTexWidth,
+                                                  colorSpace,
+                                                  info /*| kCGBitmapByteOrder32Big*/);
+        CGColorSpaceRelease(colorSpace);
+        CGContextClearRect( context, CGRectMake( 0, 0, textureInfo.mTexWidth, textureInfo.mTexHeight ) );
+        CGContextTranslateCTM( context, 0, textureInfo.mTexHeight - textureInfo.mHeight );
+        CGContextDrawImage( context, CGRectMake( 0, 0, textureInfo.mWidth, textureInfo.mHeight ), image.CGImage );
+
+        tex = new JTexture();
+
+        if (tex)
+        {
+                if (mImageFilter != NULL)
+                        mImageFilter->ProcessImage((PIXEL_TYPE*)textureInfo.mBits, textureInfo.mWidth, textureInfo.mHeight);
+
+                tex->mFilter = TEX_FILTER_LINEAR;
+                tex->mWidth = textureInfo.mWidth;
+                tex->mHeight = textureInfo.mHeight;
+                tex->mTexWidth = textureInfo.mTexWidth;
+                tex->mTexHeight = textureInfo.mTexHeight;
+                tex->mBuffer = textureInfo.mBits;
+        } else {
+                NSLog(@"JTexture for %s not created\n", filename);
+        }
+
+        CGContextRelease(context);
+    } while(0);
+
+    if(rawdata)
+        delete[] rawdata;
+
+    [image release];
+    [texData release];
+
+    return tex;
+}
+#elif (defined QT_CONFIG)
+JTexture* JRenderer::LoadTexture(const char* filename, int mode, int TextureFormat __attribute__((unused)))
+{
+  JTexture *tex = NULL;
+  int rawsize = 0;
+  BYTE* rawdata = NULL;
+  JFileSystem* fileSystem = JFileSystem::GetInstance();
+
+  do {
+    if (!fileSystem->OpenFile(filename))
+      break;
+
+    rawsize = fileSystem->GetFileSize();
+    rawdata = new BYTE[rawsize];
+
+    if (!rawdata)
+    {
+      fileSystem->CloseFile();
+      break;
+    }
+
+    fileSystem->ReadFile(rawdata, rawsize);
+    fileSystem->CloseFile();
+
+    QImage tmpImage = QImage::fromData(rawdata, rawsize);
+    if(tmpImage.isNull())
+      break;
+
+    tex = new JTexture();
+    if (tex)
+    {
+      tmpImage = tmpImage.convertToFormat(QImage::Format_ARGB32_Premultiplied);
+      tex->mImage = tmpImage.rgbSwapped();
+
+      if (mImageFilter != NULL)
+          mImageFilter->ProcessImage((PIXEL_TYPE*)tex->mImage.bits(), tex->mImage.width(), tex->mImage.height());
+
+      tex->mFilter = TEX_FILTER_LINEAR;
+      tex->mWidth = tex->mImage.width();
+      tex->mHeight = tex->mImage.height();
+      tex->mTexWidth = tex->mImage.width();
+      tex->mTexHeight = tex->mImage.height();
+
+      tex->mBuffer = tex->mImage.bits();
+    }
+  } while(false);
+
+  if(rawdata)
+    delete[] rawdata;
+
+  return tex;
+}
+#endif //IOS
+
+void JRenderer::TransferTextureToGLContext(JTexture& inTexture)
+{
+    if (inTexture.mBuffer != NULL)
+    {
+        GLuint texid;
+        checkGlError();
+        glGenTextures(1, &texid);
+        inTexture.mTexId = texid;
+        mCurrentTex = texid;
+
+        //    glError = glGetError();
+
+        if (1)///*texid*/ glError == 0)
+        {
+
+            // OpenGL texture has (0,0) at lower-left
+            // Pay attention when doing texture mapping!!!
+
+            glBindTexture(GL_TEXTURE_2D, mCurrentTex);    // Bind To The Texture ID
+
+            /* NOT USED
+            if (mode == TEX_TYPE_MIPMAP)			// generate mipmaps
+            {
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+            glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
+            glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
+            gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, textureInfo.mTexWidth, textureInfo.mTexHeight, GL_RGBA, GL_UNSIGNED_BYTE, textureInfo.mBits);
+            }
+            else if (mode == TEX_TYPE_SKYBOX)		// for skybox
+            {
+            #define GL_CLAMP_TO_EDGE	0x812F
+
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
+            gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, textureInfo.mTexWidth, textureInfo.mTexHeight, GL_RGBA, GL_UNSIGNED_BYTE, textureInfo.mBits);
+            }
+            else									// single texture
+            */
+            {
+                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+                glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
+                glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
+                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, inTexture.mTexWidth, inTexture.mTexHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, inTexture.mBuffer);
+            }
+        }
+#ifndef QT_CONFIG
+        delete [] inTexture.mBuffer;
+#endif
+        inTexture.mBuffer = NULL;
+
+        checkGlError();
+    }
+}
 
 JTexture* JRenderer::CreateTexture(int width, int height, int mode __attribute__((unused)))
 {
-	int size = width * height * sizeof(PIXEL_TYPE);			// RGBA
+  checkGlError();
+  int size = width * height * sizeof(PIXEL_TYPE);			// RGBA
 
 	BYTE* buffer = new BYTE[size];
 
@@ -2022,7 +2219,8 @@ JTexture* JRenderer::CreateTexture(int width, int height, int mode __attribute__
 
 		memset(buffer, 0, size);
 
-		glBindTexture(GL_TEXTURE_2D, texid);
+		mCurrentTex = texid;
+		glBindTexture(GL_TEXTURE_2D, mCurrentTex);
 
 		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
@@ -2030,7 +2228,8 @@ JTexture* JRenderer::CreateTexture(int width, int height, int mode __attribute__
 
 		delete[] buffer;
 
-		return tex;
+    checkGlError();
+    return tex;
 	}
 	else
 		return NULL;
@@ -2048,6 +2247,7 @@ void JRenderer::EnableVSync(bool flag __attribute__((unused)))
 
 void JRenderer::ClearScreen(PIXEL_TYPE color)
 {
+  checkGlError();
     JColor col;
     col.color = color;
 
@@ -2055,44 +2255,52 @@ void JRenderer::ClearScreen(PIXEL_TYPE color)
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
 //	FillRect(0.0f, 0.0f, SCREEN_WIDTH_F, SCREEN_HEIGHT_F, color);
+  checkGlError();
 }
 
 
 void JRenderer::SetTexBlend(int src, int dest)
 {
-	if (src != mCurrTexBlendSrc || dest != mCurrTexBlendDest)
+  checkGlError();
+  if (src != mCurrTexBlendSrc || dest != mCurrTexBlendDest)
 	{
 		mCurrTexBlendSrc = src;
 		mCurrTexBlendDest = dest;
 
 		glBlendFunc(src, dest);
 	}
+  checkGlError();
 }
 
 
 void JRenderer::SetTexBlendSrc(int src)
 {
-	if (src != mCurrTexBlendSrc)
+  checkGlError();
+  if (src != mCurrTexBlendSrc)
 	{
 		mCurrTexBlendSrc = src;
 		glBlendFunc(mCurrTexBlendSrc, mCurrTexBlendDest);
 	}
+  checkGlError();
 }
 
 
 void JRenderer::SetTexBlendDest(int dest)
 {
-	if (dest != mCurrTexBlendDest)
+  checkGlError();
+  if (dest != mCurrTexBlendDest)
 	{
 		mCurrTexBlendDest = dest;
 		glBlendFunc(mCurrTexBlendSrc, mCurrTexBlendDest);
 	}
+  checkGlError();
 }
 
 
 void JRenderer::Enable2D()
 {
-	if (mCurrentRenderMode == MODE_2D)
+  checkGlError();
+  if (mCurrentRenderMode == MODE_2D)
 		return;
 
 	mCurrentRenderMode = MODE_2D;
@@ -2111,6 +2319,7 @@ void JRenderer::Enable2D()
 #endif //(!defined GL_ES_VERSION_2_0) && (!defined GL_VERSION_2_0)
 
 	glDisable (GL_DEPTH_TEST);
+  checkGlError();
 }
 
 
@@ -2185,7 +2394,8 @@ void JRenderer::PopMatrix()
 
 void JRenderer::RenderTriangles(JTexture* texture, Vertex3D *vertices, int start, int count)
 {
-	if (texture)
+  checkGlError();
+  if (texture)
 		BindTexture(texture);
 
   int index = start*3;
@@ -2247,6 +2457,7 @@ void JRenderer::RenderTriangles(JTexture* texture, Vertex3D *vertices, int start
 		}
 	glEnd();
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
+  checkGlError();
 }
 
 
@@ -2258,10 +2469,10 @@ void JRenderer::SetFOV(float fov)
 
 void JRenderer::FillPolygon(float* x, float* y, int count, PIXEL_TYPE color)
 {
-	JColor col;
+  checkGlError();
+  JColor col;
 	col.color = color;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   int i;
   GLubyte* colors = new GLubyte[count*4];
@@ -2304,6 +2515,7 @@ void JRenderer::FillPolygon(float* x, float* y, int count, PIXEL_TYPE color)
   delete[] colors;
 
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_TRIANGLE_FAN);
 
@@ -2315,18 +2527,19 @@ void JRenderer::FillPolygon(float* x, float* y, int count, PIXEL_TYPE color)
 	glEnd();  
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 
 void JRenderer::DrawPolygon(float* x, float* y, int count, PIXEL_TYPE color)
 {
-	JColor col;
+  checkGlError();
+  JColor col;
 	col.color = color;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   int i;
   int number = count+1;
@@ -2373,6 +2586,7 @@ void JRenderer::DrawPolygon(float* x, float* y, int count, PIXEL_TYPE color)
   delete[] vVertices;
   delete[] colors;
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_LINE_STRIP);
 
@@ -2387,9 +2601,10 @@ void JRenderer::DrawPolygon(float* x, float* y, int count, PIXEL_TYPE color)
 
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 
@@ -2423,10 +2638,10 @@ void JRenderer::DrawLine(float x1, float y1, float x2, float y2, float lineWidth
 
 void JRenderer::DrawCircle(float x, float y, float radius, PIXEL_TYPE color)
 {
-	JColor col;
+  checkGlError();
+  JColor col;
 	col.color = color;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   int i, index;
   const int number = 181;
@@ -2471,6 +2686,7 @@ void JRenderer::DrawCircle(float x, float y, float radius, PIXEL_TYPE color)
 
   glDrawArrays(GL_LINE_STRIP,0,number);
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_LINE_STRIP);
 
@@ -2484,17 +2700,18 @@ void JRenderer::DrawCircle(float x, float y, float radius, PIXEL_TYPE color)
 	glEnd();
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 void JRenderer::FillCircle(float x, float y, float radius, PIXEL_TYPE color)
 {
-	JColor col;
+  checkGlError();
+  JColor col;
 	col.color = color;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   int i, index;
   const int number = 182;
@@ -2543,6 +2760,7 @@ void JRenderer::FillCircle(float x, float y, float radius, PIXEL_TYPE color)
 
   glDrawArrays(GL_TRIANGLE_FAN,0,number);
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_TRIANGLE_FAN);
 
@@ -2559,22 +2777,23 @@ void JRenderer::FillCircle(float x, float y, float radius, PIXEL_TYPE color)
 
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 
 void JRenderer::DrawPolygon(float x, float y, float size, int count, float startingAngle, PIXEL_TYPE color)
 {
-	JColor col;
+  checkGlError();
+  JColor col;
 	col.color = color;
 
 	float angle = startingAngle*RAD2DEG;
 	float steps = 360.0f/count;
 	size /= 2;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   int i;
   GLfloat* vVertices = new GLfloat[3*count];
@@ -2619,6 +2838,7 @@ void JRenderer::DrawPolygon(float x, float y, float size, int count, float start
   delete[] vVertices;
   delete[] colors;
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_LINE_LOOP);
 
@@ -2634,15 +2854,17 @@ void JRenderer::DrawPolygon(float x, float y, float size, int count, float start
 	glEnd();
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 
 void JRenderer::FillPolygon(float x, float y, float size, int count, float startingAngle, PIXEL_TYPE color)
 {
-	JColor col;
+  checkGlError();
+  JColor col;
 	col.color = color;
 
 	float angle = startingAngle*RAD2DEG;
@@ -2650,7 +2872,6 @@ void JRenderer::FillPolygon(float x, float y, float size, int count, float start
 	float steps = 360.0f/count;
 	size /= 2;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   int i;
   GLfloat* vVertices = new GLfloat[3*(count+2)];
@@ -2703,6 +2924,7 @@ void JRenderer::FillPolygon(float x, float y, float size, int count, float start
   delete[] vVertices;
   delete[] colors;
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_TRIANGLE_FAN);
 
@@ -2722,9 +2944,10 @@ void JRenderer::FillPolygon(float x, float y, float size, int count, float start
 
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 
@@ -2737,12 +2960,12 @@ void JRenderer::SetImageFilter(JImageFilter* imageFilter)
 
 void JRenderer::DrawRoundRect(float x, float y, float w, float h, float radius, PIXEL_TYPE color)
 {
-	x+=w+radius;
+  checkGlError();
+  x+=w+radius;
 	y+=h+radius;
 	JColor col;
 	col.color = color;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   int i;
   int number = 360;
@@ -2808,6 +3031,7 @@ void JRenderer::DrawRoundRect(float x, float y, float w, float h, float radius,
   delete[] vVertices;
   delete[] colors;
 #else
+  glDisable(GL_TEXTURE_2D);
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_LINE_LOOP);
 	int i;
@@ -2847,22 +3071,23 @@ void JRenderer::DrawRoundRect(float x, float y, float w, float h, float radius,
 
   // default color
   glColor4ub(255, 255, 255, 255);
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
 
-	glEnable(GL_TEXTURE_2D);
+  checkGlError();
 }
 
 
 
 void JRenderer::FillRoundRect(float x, float y, float w, float h, float radius, PIXEL_TYPE color)
 {
-	x+=w+radius;
+  checkGlError();
+  x+=w+radius;
 	y+=radius;
 
 	JColor col;
 	col.color = color;
 
-	glDisable(GL_TEXTURE_2D);
 #if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
   int i, offset;
   int number = 2+360;
@@ -2918,11 +3143,13 @@ void JRenderer::FillRoundRect(float x, float y, float w, float h, float radius,
   // Load the vertex position
   glVertexAttribPointer ( prog1_positionLoc, 3, GL_FLOAT,
                           GL_FALSE, 3 * sizeof(GLfloat), vVertices );
+
   // Load the color
   glVertexAttribPointer(prog1_colorLoc, 4, GL_UNSIGNED_BYTE,
                         GL_TRUE, 4 * sizeof(GLubyte), colors);
 
   glEnableVertexAttribArray ( prog1_positionLoc );
+
   glEnableVertexAttribArray ( prog1_colorLoc );
 
   // Load the MVP matrix
@@ -2933,6 +3160,8 @@ void JRenderer::FillRoundRect(float x, float y, float w, float h, float radius,
   delete[] vVertices;
   delete[] colors;
 #else
+  glDisable(GL_TEXTURE_2D);
+
   glColor4ub(col.r, col.g, col.b, col.a);
   glBegin(GL_TRIANGLE_FAN);
 
@@ -2979,9 +3208,9 @@ void JRenderer::FillRoundRect(float x, float y, float w, float h, float radius,
 
   // default color
   glColor4ub(255, 255, 255, 255);
+
+  glEnable(GL_TEXTURE_2D);
 #endif //#if (defined GL_ES_VERSION_2_0) || (defined GL_VERSION_2_0)
-
-	glEnable(GL_TEXTURE_2D);
-
+  checkGlError();
 }
 
