@@ -2,7 +2,6 @@
 #define _DUELLAYERS_H_
 
 #include "GuiLayers.h"
-#include "CardSelector.h"
 
 class MTGGuiHand;
 class MTGGuiPlay;
@@ -13,6 +12,7 @@ class GuiHandSelf;
 class GuiHandOpponent;
 class GuiCombat;
 class GuiAvatars;
+class CardSelectorBase;
 struct Pos;
 
 class DuelLayers {
@@ -33,6 +33,7 @@ public:
   ActionLayer * actionLayer();
   ActionStack * stackLayer();
   GuiCombat * combatLayer();
+  GuiAvatars * GetAvatars();
   void init();
   virtual void Update(float dt, Player * player);
   void CheckUserInput(int isAI);
@@ -42,7 +43,7 @@ public:
   int receiveEvent(WEvent * e);
   float RightBoundary();
 
-  CardSelector* cs;
+  CardSelectorBase* mCardSelector;
 };
 
 #include "ActionLayer.h"
