@@ -4,7 +4,6 @@
 #include <string>
 #include <JLBFont.h>
 #include <JGui.h>
-#include <string>
 #include "SimpleMenu.h"
 
 using std::string;
@@ -25,13 +24,13 @@ class SimpleMenuItem: public JGuiObject
 
  public:
   string desc;
-  SimpleMenuItem(SimpleMenu* _parent, int id, int fontId, string text, int x, int y, bool hasFocus = false, bool autoTranslate = false);
+  SimpleMenuItem(SimpleMenu* _parent, int id, int fontId, string text, float x, float y, bool hasFocus = false, bool autoTranslate = false);
 
-  int mX;
-  int mY;
+  float mX;
+  float mY;
 
-  void Relocate(int x, int y);
-  int GetWidth();
+  void Relocate(float x, float y);
+  float GetWidth();
   bool hasFocus();
 
   void RenderWithOffset(float yOffset);
@@ -42,6 +41,7 @@ class SimpleMenuItem: public JGuiObject
   virtual bool Leaving(JButton key);
   virtual bool ButtonPressed();
   virtual ostream& toString(ostream& out) const;
+  virtual bool getTopLeft(float& top, float& left) {top = mY; left = mX; return true;};
 };
 
 #endif
