@@ -5,7 +5,7 @@
 #define snprintf sprintf_s
 #endif 
 
-#if defined (WIN32) || defined (LINUX)
+#if (defined (WIN32) || defined (LINUX)) && defined (_DEBUG)
 #define TESTSUITE 1
 
 #else
